function [fz, z, grad] = Gradient_mit_Armijo(f, gradf, z0, beta, gamma, tol)
% Realisiert das Gradientenverfahren mit der Armijo-Schrittweitenregel
% (vgl. Algorithmus 2 im Skript)
% Hier aber ohne symbolisch definierte Funktion!

% initialisiere
continue_loop = 1;
i = 1;
% ersten Funktionswert berechnen und speichern
z(:,i) = z0;
fz(i,1) = f(z(:,i));
% Gradient berechnen
grad(:,i) = gradf(z(:,i));
while continue_loop
    % falls der Gradient numerisch null ist wird abgebrochen. Wichtig: ein check auf == 0 wäre fatal.
    if norm(grad(:,i)) < tol
        continue_loop = 0;
    else
        % starte immer mit Schrittweite 1
        s = 1;
        % gehe einen Schritt und berechne Funktionswert
        z_new = z(:,i) - s*grad(:,i);
        fz_new = f(z_new);
        % solange das Armijo-Kriterium nicht erfüllt ist
        while fz_new > fz(i,1) - gamma*s*norm(grad(:,i))^2% alternativ zu grad(:,i)'*grad(:,i)
            % verringere Schrittweite
            s = s*beta;
            % gehe Schritt
            z_new = z(:,i) - s*grad(:,i);
            % berechne Funktionswert
            fz_new = f(z_new);
        end
        % wenn Armijo-Kriterium erfüllt - vollziehe den Schritt
        i = i+1;
        % speichere output
        z(:,i) = z_new;
        fz(i,1) = fz_new;
        % berechne Gradient an neuem Punkt
        grad(:,i) = gradf(z(:,i));
        disp(norm(grad(:,i))); % optionaler output: damit man abschätzen kann, wie lange das Verfahren noch braucht
    end
end